function [y,ar] = preprocess(x,Fe,K,affich);

% function [y,ar] = preprocess(x,Fe,K,affich);
% pre-processing for an audio signal
% median filtering of the spectrum + accentuation;
% y: filtered signal
% ar: AR coeff. of the estimated filter (H(z) = 1/AR(z))
% x: signal
% Fe: sampling frequency
% K: prediction order (default=12)
% affich = 1/0: with or without display (0=default)
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 3,affich=0;K=12;end
if nargin < 4,affich=0;end

warning off
nfft = 512;

dyn = 80; % dyn max de la dsp en dB (sur le filtrage m?dian)

% estimation de la densit? de puissance
[X,w] = psd(x,nfft,Fe);

if affich
figure;    
plot(w,10*log10(abs(X)),'--','color',[.5 .5 .5]);
hold on
end

Xm = medfiltp(abs(X),61,33);Xm = Xm(:);
ind = find(10*log10(Xm) < max(10*log10(Xm))-dyn);
Xm(ind) = max(Xm)/10^(dyn/10);


Xm = [Xm; Xm( end-1:-1:2)];
gam = real(ifft(Xm));
gam = gam(1:nfft/2);gam = gam(:);

Rxx = toeplitz( gam(1:K),gam(1:K));
a = -pinv(Rxx)*gam(2:K+1);
sig = gam(1:K+1).'*[1;a];
if affich
[H,w] = freqz(1,[1;a],nfft,1);
plot(w*Fe,20*log10(sqrt(sig) * H),'y');
end

y = filter([1;a],1,x);

if affich
[Y,w] = psd(y,1024,Fe);
plot(w,10*log10(abs(Y)),'r');
end

ar = [1;a];

