function [A, F] = tone2peaks(fundamentalFrequency, amplitude, numberOfHarmonics, duration, sr)
%% generate an artificial peaks representation of a tone
%% fundamentalFrequency : the fundamental frequency of the tone (in Hz)
%% numberOfHarmonics : the number of harmonics of the tone
%% duration : the length of the tone (in frames)
%% amplitude : the amplitude of the tone
%% sr : sampling rate
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin<1, fundamentalFrequency = 440; end
if nargin<2, amplitude = 1; end
if nargin<3, numberOfHarmonics=10; end
if nargin<4, duration=20; end
if nargin<5, sr=44100; end

A = size(numberOfHarmonics, duration);
F = size(numberOfHarmonics, duration);
for k=1:duration
    for l=1:numberOfHarmonics
        F(l, k) = fundamentalFrequency*l;
        A(l, k) = amplitude*(numberOfHarmonics-l)/numberOfHarmonics;
    end
end










