function [a, mu, phi, omega, psi] = derivative (x, d1, d2, Fs, m)

% see
% Sylvain Marchand and Philippe Depalle,
% Generalization of the Derivative Analysis Method to Non-Stationary Sinusoidal Modeling,
% In Proceedings of the International Conference on Digital Audio Effects (DAFx),
% pages 281-288, Espoo, Finland, September 2008.
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

  N = length (x);
  w = hann_window (N);
  X = fft (w .* x);
  D1 = fft (w .* d1);
  D2 = fft (w .* d2);
  if nargin < 5
    % by default, consider the global maximum for the (strictly) positive frequencies
    [v, m] = max (abs (X(2:N/2)));
    m = m + 1;
  end
  r1 = D1(m) ./ X(m);
  r2 = D2(m) ./ X(m);
  omega = imag (r1);  % should use this frequency then...
  [X, X1, X2] = compute_spectra (N, Fs, w.*x, w.*d1, w.*d2, omega);
  assert (length(find(X==0)) == 0);  % zeros should never happen...
  r1 = X1 ./ X;
  r2 = X2 ./ X;
  mu = real (r1);
  psi = imag(r2) - 2*mu.*omega;
  p = Gamma (N, Fs, zeros(size(X)), mu, psi, w);
  phi = angle (X ./ p);
  a = abs (X ./ p);

end
