function test_derivative ()

% see Figure 1 of
% Sylvain Marchand and Philippe Depalle,
% Generalization of the Derivative Analysis Method to Non-Stationary Sinusoidal Modeling,
% In Proceedings of the International Conference on Digital Audio Effects (DAFx),
% pages 281-288, Espoo, Finland, September 2008.
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

  test_init;

  clip_boundaries = 1;

  Fs = 2000;  % sampling rate in Hz
  N = 4000;  % frames size in samples
  F = Fs/2;  % test up to the Nyquist frequency
  inc = 5;  % frequency increment

  close all;

  % first derivative

  err1 = 0;
  err2 = 0;
  for f = (1:inc:F-1)
    t = [0:N-1]/Fs;
    src = sin (2*pi*f*t);
    the = 2*pi*f * cos (2*pi*f*t);
    drv = discrete_derivative (src, Fs, clip_boundaries);
    ref = [diff(src)*Fs, 0];
    the = resize_signal (the, drv);
    ref = resize_signal (ref, drv);
    the = the(1:end-1);
    drv = drv(1:end-1);
    ref = ref(1:end-1);
    err1(f) = max (abs (ref - the));
    err2(f) = max (abs (drv - the));
  end
  x = 2*pi*[1:inc:F-1]/Fs;
  % replace err1 by its model, to avoid spurious discontinuities
  p = polyfit (x, err1(1:inc:end)./(x*Fs), 3);
  err = polyval (p, x);
  plot (x(1:5:end), log10(err(1:5:end)), 'k*', x, log10(err2(1:inc:end)./(x*Fs)), 'k-');
  title ('approximation of the first derivative', 'FontName', 'Times-Roman', 'FontSize', 20);
  xlabel ('frequency of the sinusoid (rad)', 'FontName', 'Times-Roman', 'FontSize', 20);
  ylabel ('maximal relative error', 'FontName', 'Times-Roman', 'FontSize', 20);
  ylabel ('maximal relative error (log10 scale)', 'FontName', 'Times-Roman', 'FontSize', 20);
  legend ('difference', 'resampling');
  legend ('difference', 'differentiator');
  legend ('location', 'northwest');
  legend ('location', 'southeast');

  print -deps 'd1-error.eps';

  figure;

  % second derivative

  err1 = 0;
  err2 = 0;
  for f = (1:inc:F-1)
    t = [0:N-1]/Fs;
    src = sin (2*pi*f*t);
    the = -(2*pi*f)^2 * sin (2*pi*f*t);
    drv = discrete_derivative (src, Fs, clip_boundaries);
    drv = discrete_derivative (drv, Fs, clip_boundaries);
    ref = [diff(diff(src))*(Fs^2), 0, 0];
    the = resize_signal (the, drv);
    ref = resize_signal (ref, drv);
    the = the(1:end-2);
    drv = drv(1:end-2);
    ref = ref(1:end-2);
    err1(f) = max (abs (ref - the));
    err2(f) = max (abs (drv - the));
  end
  x = 2*pi*[1:inc:F-1]/Fs;
  % replace err1 by its model, to avoid spurious discontinuities
  p = polyfit (x, err1(1:inc:end)./((x*Fs).^2), 5);
  err = polyval (p, x);
  plot (x(1:5:end), log10(err(1:5:end)), 'k*', x, log10(err2(1:inc:end)./((x*Fs).^2)), 'k-');
  title ('approximation of the second derivative', 'FontName', 'Times-Roman', 'FontSize', 20);
  xlabel ('frequency of the sinusoid (rad)', 'FontName', 'Times-Roman', 'FontSize', 20);
  ylabel ('maximal relative error', 'FontName', 'Times-Roman', 'FontSize', 20);
  ylabel ('maximal relative error (log10 scale)', 'FontName', 'Times-Roman', 'FontSize', 20);
  legend ('difference', 'resampling');
  legend ('difference', 'differentiator');
  legend ('location', 'northwest');
  legend ('location', 'southeast');

  print -deps 'd2-error.eps';

end
