function [a_crb, mu_crb, phi_crb, omega_crb, psi_crb] = crb (sigma, a, mu_set, psi_set, N)

% see Zhou et al.,
% On Polynomial Phase Signal with Time-Varying Amplitudes,
% IEEE Trans. Sig. Proc. 44(4):848-861, 1996;
% and Djuric and Kay,
% Parameter Estimation of Chirp Signals,
% IEEE Trans. Acous., Speech, and Sig. Proc. 38(12):2118-2126, 1990.
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

  nofm = (psi_set == 0);  % the bounds change if there is no FM
  % return the expectation (mean) of the bounds in case of different AM
  init = zeros (length(mu_set), length(sigma));
  a_crb = init;
  mu_crb = init;
  phi_crb = init;
  omega_crb = init;
  psi_crb = init;
  for idx = 1:length(mu_set)
    mu = mu_set(idx);
    e0 = crb_epsilon (0, -mu, N);
    e1 = crb_epsilon (1, -mu, N);
    e2 = crb_epsilon (2, -mu, N);
    e3 = crb_epsilon (3, -mu, N);
    e4 = crb_epsilon (4, -mu, N);
    % amplitude, amplitude modulation
    a_crb(idx,:) = (sigma.^2*e2) / (2*(e0*e2-e1^2));
    mu_crb(idx,:) = (sigma.^2*e0) / (2*a^2*(e0*e2-e1^2)*N^2);
    % phase, frequency, frequency modulation
    D = e0*e2*e4 - e1^2*e4 - e0*e3^2 + 2*e1*e2*e3 - e2^3;
    if nofm
      phi_crb(idx,:) = (sigma.^2*e2) / (2*a^2*(e0*e2-e1^2));
      omega_crb(idx,:) = (sigma.^2*e0) / (2*a^2*N^2*(e0*e2-e1^2));
    else
      phi_crb(idx,:) = (sigma.^2*(e2*e4-e3^2)) / (2*a^2*D);
      omega_crb(idx,:) = (sigma.^2*(e0*e4-e2^2)) / (2*a^2*N^2*D);
    end
    psi_crb(idx,:) = (sigma.^2*(e0*e2-e1^2)) / (2*a^2*N^4*D)*4;
  end
  a_crb = mean (a_crb, 1);
  mu_crb = mean (mu_crb, 1);
  phi_crb = mean (phi_crb, 1);
  omega_crb = mean (omega_crb, 1);
  psi_crb = mean (psi_crb, 1);

end
