function result = Gamma (N, Fs, Delta, mu, psi, w)

% see Equation (4) of
% Sylvain Marchand and Philippe Depalle,
% Generalization of the Derivative Analysis Method to Non-Stationary Sinusoidal Modeling,
% In Proceedings of the International Conference on Digital Audio Effects (DAFx),
% pages 281-288, Espoo, Finland, September 2008.
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

  L = size (Delta, 2);
  Mw = repmat (w, L, 1);
  t = time_axis (N, Fs);
  result = transpose (sum (Mw.*exp(mu.'*t+i*(Delta.'*t+psi.'*(t.^2)/2)),2));
  assert(length(find(result==0))==0);  % zeros should never happen...
  idx = [find(isnan(result)), find(isinf(result))];  % in case of problems with large mu value...
  result(idx) = 1;  % ...reset to default: no correction

end
