function featuresData = computeFeaturesPerFile(fileName, features, config, ap, degradationHandle)
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin<5
    degradationHandle={};
end

%open the file
if config.verbose>1
    fileName
end




[a, fs] = wavread(fileName);

if(fs ~= ap.sr)
    %disp('wrong sampling frequency');
    ap = getAnalysisParameters(fs);
end

if size(a, 2)>1
    a=a(:, 1);
end

%degrade if required
if config.degradationId~=0 && config.degradationId<4
    a = feval(degradationHandle{1}, a, ap.sr, config.degradationLevel);
end

% compute spectrogram (root of every features)
if ap.preEmphasis
    a = filter([1 -ap.preEmphasis], 1, a);
end
if ap.dcBlock
    a= filter([1 -1], [1 -ap.dcBlock], a);
end
% no zeros
a = a+eps*rand(length(a), 1);

%sound(a, fs)
if ~isempty(features.longtermFeaturesId)
    [F, A] = ifgram(a, ap.frameSize, ap.frameSize, ap.hopSize,  ap.frameSize);
else
    A = spectrogram(a, hamming(ap.frameSize), ap.frameSize-ap.hopSize,  ap.frameSize);
end
Z=angle(A);
A=abs(A);

if config.degradationId>3
    A = feval(degradationHandle{1}, A, ap.sr, config.degradationLevel);
end

if 0
    %% saving degraded version of audio
    AZ = A.*exp(i*Z);
    AZZ = flipud(conj(AZ));
    AZ = [AZ; AZZ];
    z=invspectrogram(AZ, ap.hopSize, ap.frameSize);
    %sound(z, fs)
    outputFileName = regexprep(config.featureFileName, 'features', 'demos/timbralSimilarity');
    outputFileName = regexprep(outputFileName, 'mat', 'wav')
    [path fileName ext] = fileparts(fileName);
    outputFileName = regexprep(outputFileName, 'solos', ['solos' fileName])
    wavwrite(z, fs, outputFileName);
end


[path name ext] = fileparts(fileName);
onsetCurveFileName = regexprep(config.onsetCurveFileName, 'johndoe', name);

instruments = getInstrumentType(config.database);
nbInstruments = length(instruments);


assumedFs = 44100;
ap = getAnalysisParameters(assumedFs);

features = getFeatures(ap, config.featureFileName, config.featuresId, instruments, nbInstruments);
timeLine = getTimeLine(config.integrationId, ap, size(A, 2), onsetCurveFileName, fs);

if max(timeLine)> size(A, 2)+1
    timeLine = timeLine(timeLine<= size(A, 2));
    timeLine = [timeLine size(A, 2)+1];
end

% perform partial tracking
if ~isempty(features.longtermFeaturesId)
    B = longtermModeling(A, F, ap, timeLine);
end

normA = sum(A, 1);
normA = repmat(normA, size(A, 1), 1);
A = A./normA;


% early integration
if config.integrationId ~= 0
    C = performIntegration(A, timeLine);
else
    C = A;
end

%compute feature set
featuresData = zeros(features.vectorLength(end), size(C, 2));
%% loop through all frames of a given file
for m=1:length(features.handles)
    %    getFeatureType(config.featuresId(m))
    if sum(features.metaFeaturesId==m)
        % features are computed from  preceding feature
        res = features.handles{m}(res, ap);
    elseif sum(features.longtermFeaturesId==m)
        % features are computed from  long term models
        res = features.handles{m}(B, ap);
    elseif sum(features.lateIntegrationFeaturesId==m)
        % features are computed from the spectrogram
        res = features.handles{m}(A, ap);
    else
        % features are comptued from the integrated spectrogram
        res = features.handles{m}(C, ap);
    end

    if sum(features.lateIntegrationFeaturesId==m)
        R = performIntegration(res, timeLine);
    else
        R = res;
    end

    featuresData(features.vectorLength(m)+1:features.vectorLength(m+1), :) = R;
end

% track nan values
nanIndicator = (1:size(C, 2));
nanDetected = 0;
for m=1:size(C, 2)
    if sum(isnan(featuresData(:, m)))
        featuresData(:, m)= rand(1, size(featuresData, 1))*10*eps;
        nanIndicator(m) = 0;
        nanDetected = 1;
    end
end
%if nanDetected
%    fprintf('nan resulting of operation of file %s', fileName);
%    featuresData = featuresData(:, nanIndicator(nanIndicator>0));
%end
