function [l,s,X2,lambda] = EM(Y, Nfft, u, h, J,lambda,l,F0,Fs);
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

I = length(Y);
K = size(u,2);
h0 = 1 - h;
bin = dec2bin(0:2^J-1);
b = zeros(2^J,J);
for k=1:2^J,
    temp = bin(k,:); b(k,:) = str2num(temp(:));
end;
e = [b,ones(2^J,1)];

% Fake 1st EM iteration

% M-Step : prior probabilities
pivar = eps + h(:,l)*(1-2*eps);
pi2 = exp( log(pivar)*b' + log(1-pivar)*(1-b)' );

% M-Step : envelope coefficients
model = 'MA';
M = 5;
sigma2 = [ones(1,J),10^(-40/20)];
A = eye(M,1)*ones(1,1+J);
B = [1,1,1,1,1;0.9,0.9,0.9,0.9,0;0.8,0,0.8,0.8,0;0.7,0,0.7,0,0;0,0,0.6,0,0]*diag([10^(-30/20)/3.4,10^(-10/20)/1.9,10^(-20/20)/4,10^(0/20)/2.7,10^(-60/20)]);

s = ones(I,1) * sigma2; % spectres des composantes (sur chaque colonne)
s2 = s*e';
k = 0;
olds2 = ones(size(s2));
while max(max(abs(log(s2)-log(olds2)))) > 1e-2 & (k < 30),
    k=k+1;
    olds2 = s2;
    spectrum1MA = diag(Y) * ((pi2 ./ s2.^2) *e);
    spectrum2MA =((pi2 ./ s2) *e);
    if model == 'MA',
        R1 = real(ifft(spectrum1MA));
        R2 = real(ifft(spectrum2MA));
    else, % if model == 'AR'
        R1 = real(ifft(spectrum2MA .* s.^2));
        R2 = real(ifft(spectrum1MA .* s.^2));
    end;
    for l=1:1+J,
        R10 = toeplitz(R1(1:M,l));
        R20 = toeplitz(R2(1:M,l));
        A(:,l) = pinv(R20) * R10 * A(:,l);
        racines = roots(A(:,l));
        I0 = find(abs(racines)>1);
        racines(I0) = conj(1./racines(I0));
        A(:,l) = poly(racines)';
    end;
    sigma2 = sigma2  .* sum(spectrum1MA .* s) ./ sum(spectrum2MA .* s);
    if model == 'MA',
        s = abs(exp(-i*2*pi*(0:I-1)'*(0:M-1)/Nfft) * A).^2 * diag(sigma2); % spectres des composantes (sur chaque colonne)
    else, % if model == 'AR'
        s = 1 ./ abs(exp(-i*2*pi*(0:I-1)'*(0:M-1)/Nfft) * A).^2 * diag(sigma2); % spectres des composantes (sur chaque colonne)
    end;
    s2 = s * e'; % spectres des mlanges (sur chaque colonne)
end;

   
% EM Algorithm
L = [0 1];
while 1; % abs(log(L(end))-log(L(end-1))) > 1e-4,
    % E-Step
    gamma2 = log(pi2) - log(s2) - diag(Y) * (1 ./ s2);
    L = [L, sum( log( sum(exp(gamma2),2) ) )];
    gamma2 = exp( gamma2 - max(gamma2,[],2)*ones(1,2^J) );
    gamma2 = diag(1 ./ (sum(gamma2,2))) * gamma2;
    gamma = gamma2 * b;

    % M-Step : prior probabilities
    pi1 = diag(1./(sum(h)) ) * h'  * gamma;
    pi0 = diag(1./(sum(h0))) * h0' * gamma;
    pi1 = max(pi1,eps); pi1 = min(pi1,1-eps);
    pi0 = max(pi0,eps); pi0 = min(pi0,1-eps);
    Q1 = pi1.*log(pi1) + (1-pi1).*log(1-pi1);
    Q0 = pi0.*log(pi0) + (1-pi0).*log(1-pi0);
    Q = diag(sum(h)) * Q1 + diag(sum(h0)) * Q0;
    [bidon,l] = max(Q);
    pi1 = diag(pi1(l,:));
    pi0 = diag(pi0(l,:));
    pivar = h(:,l)*diag(pi1) + h0(:,l) * diag(pi0);
    pi2 = exp( log(pivar)*b' + log(1-pivar)*(1-b)' );

    figure(1);
    subplot(4,1,1);    plot(F0,Q(:,1),'.-'); title('(a)'); ylabel('Q_{l,1}'); rect = [min(F0),3000,min(Q(:,1))*1.2,0]; axis(rect);
    subplot(4,1,2);    plot(F0,Q(:,2),'.-'); title('(b)'); ylabel('Q_{l,2}'); rect = [min(F0),3000,min(Q(:,2))*1.2,0]; axis(rect);
    subplot(4,1,3);    plot(F0,Q(:,3),'.-'); title('(c)'); ylabel('Q_{l,3}'); rect = [min(F0),3000,min(Q(:,3))*1.2,0]; axis(rect);
    subplot(4,1,4);    plot(F0,Q(:,4),'.-'); title('(d)'); ylabel('Q_{l,4}'); rect = [min(F0),3000,min(Q(:,4))*1.2,0]; axis(rect);
    xlabel('Fundamental frequency candidates')
    disp(F0(l)');

    % M-Step : envelope coefficients
    olds2 = ones(size(s2));
    k=0;
    %while (max(max(abs(log(s2)-log(olds2)))) > 1e-2) & (k < 30),
        k=k+1;
        olds2 = s2;
        spectrum1MA = diag(Y) * ((gamma2 ./ s2.^2) *e);
        spectrum2MA =((gamma2 ./ s2) *e);
        if model == 'MA',
            R1 = real(ifft(spectrum1MA));
            R2 = real(ifft(spectrum2MA));
        else, % if model == 'AR'
            R1 = real(ifft(spectrum2MA .* s.^2));
            R2 = real(ifft(spectrum1MA .* s.^2));
        end;
        for l=1:1+J,
            R10 = toeplitz(R1(1:M,l));
            R20 = toeplitz(R2(1:M,l));
            A(:,l) = pinv(R20) * R10 * A(:,l);
            racines = roots(A(:,l));
            I0 = find(abs(racines)>1);
            racines(I0) = conj(1./racines(I0));
            A(:,l) = poly(racines)';
        end;
        sigma2 = sigma2  .* sum(spectrum1MA .* s) ./ sum(spectrum2MA .* s);
        if model == 'MA',
            s = abs(exp(-i*2*pi*(0:I-1)'*(0:M-1)/Nfft) * A).^2 * diag(sigma2); % spectres des composantes (sur chaque colonne)
        else, % if model == 'AR'
            s = 1 ./ abs(exp(-i*2*pi*(0:I-1)'*(0:M-1)/Nfft) * A).^2 * diag(sigma2); % spectres des composantes (sur chaque colonne)
        end;
        s2 = s * e'; % spectres des mlanges (sur chaque colonne)
    %end; 

    % Affichage
    X2 = diag(Y) * (pi2./(s2.^2)*e) .* (s.^2);
    data = 10*log10(X2);
s0 = abs(exp(-i*2*pi*(0:I-1)'*(0:M-1)/Nfft) * B).^2;
figure(2);
subplot(J+2,1,1);
plot((0:I/2-1)/I*Fs,10*log10(Y(1:I/2)),'b');
rect = [0,Fs/2,-80,0]; axis(rect);
ylabel('Y_i (dB)');
title('(a)')
titre = ['(b)';'(c)';'(d)';'(e)';'(f)'];
    for j=1:J+1,
        subplot(J+2,1,j+1);
        plot((0:I/2-1)/I*Fs,10*log10(s0(1:I/2,j)),'--r'); hold on;
        plot((0:I/2-1)/I*Fs,data(1:I/2,j),'b'); hold off;
        rect = [0,Fs/2,-80,0]; axis(rect);
        title(titre(j,:));
        ylabel(sprintf('%s%d%s','|{X}_{i,',j,'}|^2 (dB)'));
    end;
    ylabel('|{N}_i|^2 (dB)');
    xlabel('Frequency (Hz)');
    pause;
end;